@@ -25,6 +25,7 @@ ACCESS_LEVEL_GUEST = 10
 ACCESS_LEVEL_REPORTER = 20
 ACCESS_LEVEL_DEVELOPER = 30
 ACCESS_LEVEL_MASTER = 40
+ACCESS_LEVEL_OWNER = 50
 
 # Maximum 'per_page' value allowed by GitLab when listing
 _MAX_PER_PAGE = 100
@@ -132,8 +133,14 @@ def _add_find_fn(api, name, parent):
             del kwargs['find_all']
         except KeyError:
             find_all = False
+        try:
+            query_data = {}
+            query_data['sudo'] = kwargs['sudo']
+            del kwargs['sudo']
+        except KeyError:
+            pass
         if not objects:
-            objects = _query_list(api, parent, {})
+            objects = _query_list(api, parent, query_data)
 
         return _find_matches(objects, kwargs, find_all)
     setattr(parent, 'find_' + name, fn)
@@ -141,10 +148,11 @@ def _add_find_fn(api, name, parent):
 
 def _add_get_fn(api, name, parent):
     """Create a <PARENT_API>.get_<name>() function"""
+    fixed_url = api._q_url.replace('merge_requests', 'merge_request')
     def fn(key=[], **kwargs):
         if key != []:
             key = [key]
-        data = parent._get(api._q_url, addl_keys=key, data=kwargs)
+        data = parent._get(fixed_url, addl_keys=key, data=kwargs)
         ret = api(parent, data)
         return ret
     setattr(parent, 'get_' + name, fn)
@@ -181,10 +189,11 @@ def _add_create_fn(api, api_definition, parent):
 
 def _add_edit_fn(api, name, parent):
     """Create <PARENT_API>.update_name(obj) and <API>.save() functions"""
+    fixed_url = api._q_url.replace('merge_requests', 'merge_request')
     def parent_fn(obj):
         return obj.save()
     def self_fn(self):
-        return self._put(api._q_url, data=self._get_data())
+        return self._put(fixed_url, data=self._get_data())
     setattr(parent, 'update_' + name, parent_fn)
     setattr(api, 'save', self_fn)
 
@@ -213,12 +222,13 @@ def _get_http_request_fn(api, method):
 def _add_extra_fn(api, action_def, parent=None):
     required_params = action_def.required_params
     # url_params are required params, but get passed as part of url
-    url_params = action_def.url_params
+    url_params = re.findall(':(\w+)', action_def.url)
     http_method = action_def.method
     url = api._q_url  # XXX: do any extra fns need unqualified url?
     url += action_def.url
     req_fn = _get_http_request_fn(api, http_method)
     num_req_params = len(url_params) + len(required_params) + 1  # + _self
+    url = url.replace('merge_requests', 'merge_request')
 
     def fn(*args, **kwargs):
         if len(args) != num_req_params:
@@ -252,6 +262,8 @@ def _add_api(definition, parent):
     else:
         uq_url = parent._uq_url
     uq_url += re.sub(r'/\:.*', '', url)  # "unqualify" the url
+    # 'sudo' is an optional parameter for all functions
+    definition.optional_params.append('sudo')
 
     cls_attrs = {
         '_key_name': definition.key_name,
@@ -429,13 +441,11 @@ class GitLab(_GitLabAPI):
     """A GitLab API connection."""
 
     def __init__(self, gitlab_url, token=None, convert_dates=True,
-                 ssl_verify=None, ssl_cert=None):
+                 ssl_verify=True, ssl_cert=None):
         setattr(_GitLabAPI, '_base_url', gitlab_url + "/api/v3")
         setattr(_GitLabAPI, '_headers', {'PRIVATE-TOKEN': token})
         setattr(_GitLabAPI, '_convert_dates_enabled', convert_dates)
-        requests_kwargs = {}
-        if ssl_verify is not None:
-            requests_kwargs['verify'] = ssl_verify
+        requests_kwargs = { 'verify': ssl_verify }
         if ssl_cert is not None:
             requests_kwargs['cert'] = ssl_cert
         setattr(_GitLabAPI, '_requests_kwargs', requests_kwargs)
@@ -464,3 +474,14 @@ class GitLab(_GitLabAPI):
         headers = {'PRIVATE-TOKEN': ret['private_token']}
         setattr(_GitLabAPI, '_headers', headers)
         return True
+
+    class sudo:
+        """Alternative sudo usage. To be used with the 'with' statement"""
+        def __init__(self, username_or_id):
+            self.user = username_or_id
+        def __enter__(self):
+            headers = getattr(_GitLabAPI, '_headers')
+            headers['SUDO'] = self.user
+        def __exit__(self, type, value, traceback):
+            headers = getattr(_GitLabAPI, '_headers')
+            del headers['SUDO']
