@@ -49,7 +49,6 @@ class APIDefinition(object):
 class ExtraActionDefinition(object):
     """Definition of a "non-standard" function"""
     url = ''
-    url_params = []
     required_params = []
     optional_params = []
 
@@ -68,6 +67,15 @@ class SSHKey(APIDefinition):
     ]
 
 
+class Member(APIDefinition):
+    url = '/members/:user_id'
+    actions = [ _LIST, _GET, _ADD, _EDIT, _DELETE ]
+    required_params = [
+        'user_id',
+        'access_level',
+    ]
+
+
 class CurrentUser(APIDefinition):
     url = '/user'
     actions = [ _GET ]
@@ -76,12 +84,14 @@ class CurrentUser(APIDefinition):
 
 class Group(APIDefinition):
     url = '/groups/:id'
-    actions = [ _LIST, _GET, _ADD ]
+    actions = [ _LIST, _GET, _ADD, _DELETE ]
     required_params = [
         'name',
         'path',
     ]
 
+    ####
+    # Extra Actions
     class TransferProjectAction(ExtraActionDefinition):
         """gl.Group.transfer_project(project_id)"""
         url = '/projects/:project_id'
@@ -91,6 +101,12 @@ class Group(APIDefinition):
         ]
     extra_actions = [ TransferProjectAction ]
 
+    ###
+    # Sub APIs
+    class Member(Member):  # Group API has limited Member actions...
+        actions = [ _LIST, _ADD, _DELETE ]
+    sub_apis = [ Member ]
+
 
 class SystemHook(APIDefinition):
     url = '/hooks/:id'
@@ -110,15 +126,6 @@ class Issue(APIDefinition):
     actions = [ _LIST ]
 
 
-class Member(APIDefinition):
-    url = '/members/:user_id'
-    actions = [ _LIST, _GET, _ADD, _EDIT, _DELETE ]
-    required_params = [
-        'user_id',
-        'access_level',
-    ]
-
-
 class Note(APIDefinition):
     url = '/notes/:note_id'
     actions = [ _LIST, _GET, _ADD ]
@@ -129,7 +136,7 @@ class Note(APIDefinition):
 
 class Project(APIDefinition):
     url = '/projects/:id'
-    actions = [ _LIST, _GET, _ADD ]
+    actions = [ _LIST, _GET, _ADD, _DELETE ]
     required_params = [
         'name',
     ]
@@ -150,20 +157,14 @@ class Project(APIDefinition):
         """gl.Project.fork_from(forked_from_id)"""
         url = '/fork/:forked_from_id'
         method = _HTTP_POST
-        url_params = [
-            'forked_from_id',
-        ]
     class DeleteForkAction(ExtraActionDefinition):
         """gl.Project.delete_fork()"""
         url = '/fork'
         method = _HTTP_DELETE
     class GetBlobAction(ExtraActionDefinition):
         """gl.Project.get_blob()"""
-        url = '/repository/commits/:sha/blob'
+        url = '/repository/commits/:sha_or_ref_name/blob'
         method = _HTTP_GET
-        url_params = [
-            'sha_or_ref_name',
-        ]
         required_params = [
             'filepath',
         ]
@@ -171,9 +172,6 @@ class Project(APIDefinition):
         """gl.Project.protect_branch()"""
         url = '/repository/branches/:branch/protect'
         method = _HTTP_PUT
-        url_params = [
-            'branch',  # GitLab API is branch name
-        ]
         @staticmethod
         def wrapper(extra_action_fn, parent):
             """Accept a Branch object instead of a branch name"""
@@ -189,9 +187,6 @@ class Project(APIDefinition):
         """gl.Project.unprotect_branch()"""
         url = '/repository/branches/:branch/unprotect'
         method = _HTTP_PUT
-        url_params = [
-            'branch',  # GitLab API is branch name
-        ]
         @staticmethod
         def wrapper(extra_action_fn, parent):
             """Accept a Branch object instead of a branch name"""
@@ -250,7 +245,7 @@ class Project(APIDefinition):
             @classmethod
             def wrapper(cls, extra_action_fn, parent):
                 def wrapped(self):
-                    extra_action_fn(self, state_event=cls.name())
+                    extra_action_fn(self, cls.name())
                     setattr(self, 'state', cls._state_after)
                 return wrapped
         class ReopenAction(CloseAction):
@@ -300,9 +295,9 @@ class Project(APIDefinition):
         required_params = [
             'source_branch',
             'target_branch',
-            'assignee_id', # TODO: is this required or not?
             'title',
         ]
+        optional_params = [ 'assignee_id' ]
 
         class PostCommentAction(ExtraActionDefinition):
             """gl.Project.MergeRequest.post_comment(note)"""
@@ -415,6 +410,8 @@ class User(APIDefinition):
         'extern_uid',
         'provider',
         'bio',
+        'admin',
+        'can_create_group',
     ]
 
     class SSHKey(SSHKey):
@@ -447,9 +444,6 @@ class GitLab(APIDefinition):
         """gl.add_project_for_user(user_id, name)"""
         url = '/projects/user/:user_id'
         method = _HTTP_POST
-        url_params = [
-            'user_id',
-        ]
         required_params = [
             'name',
         ]
@@ -471,8 +465,27 @@ class GitLab(APIDefinition):
                 project_data = extra_action_fn(*args, **kwargs)
                 return gitlab3.Project(parent, project_data)
             return wrapped
+    class FindProjectsByNameAction(ExtraActionDefinition):
+        """gl.find_projects_by_name(query)"""
+        url = '/projects/search/:query'
+        method = _HTTP_GET
+        optional_params = [
+            'per_page',
+            'page',
+        ]
+        @staticmethod
+        def wrapper(extra_action_fn, parent):
+            def wrapped(*args, **kwargs):
+                """Return a list of Projects"""
+                import gitlab3
+                ret = []
+                projects = extra_action_fn(*args, **kwargs)
+                for project_data in projects:
+                    ret.append(gitlab3.Project(parent, project_data))
+                return ret
+            return wrapped
 
-    extra_actions = [ AddProjectForUserAction ]
+    extra_actions = [ AddProjectForUserAction, FindProjectsByNameAction ]
 
     sub_apis = [
         CurrentUser,
